"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .bankaccount import BankAccount, BankAccountTypedDict
from .currency import Currency
from .customfield import CustomField, CustomFieldTypedDict
from .custommappings import CustomMappings, CustomMappingsTypedDict
from .email import Email, EmailTypedDict
from .linkedledgeraccount import LinkedLedgerAccount, LinkedLedgerAccountTypedDict
from .linkedledgeraccount_input import (
    LinkedLedgerAccountInput,
    LinkedLedgerAccountInputTypedDict,
)
from .linkedparentcustomer import LinkedParentCustomer, LinkedParentCustomerTypedDict
from .linkedtaxrate import LinkedTaxRate, LinkedTaxRateTypedDict
from .linkedtaxrate_input import LinkedTaxRateInput, LinkedTaxRateInputTypedDict
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from .website import Website, WebsiteTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from enum import Enum
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class CustomerStatusStatus(str, Enum):
    r"""Customer status"""

    ACTIVE = "active"
    INACTIVE = "inactive"
    ARCHIVED = "archived"
    GDPR_ERASURE_REQUEST = "gdpr-erasure-request"
    UNKNOWN = "unknown"


class CustomerTypedDict(TypedDict):
    id: str
    r"""A unique identifier for an object."""
    downstream_id: NotRequired[Nullable[str]]
    r"""The third-party API ID of original entity"""
    display_id: NotRequired[Nullable[str]]
    r"""Display ID"""
    display_name: NotRequired[Nullable[str]]
    r"""Display name"""
    company_name: NotRequired[Nullable[str]]
    r"""The name of the company."""
    company_id: NotRequired[Nullable[str]]
    r"""The company or subsidiary id the transaction belongs to"""
    title: NotRequired[Nullable[str]]
    r"""The job title of the person."""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the person."""
    middle_name: NotRequired[Nullable[str]]
    r"""Middle name of the person."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the person."""
    suffix: NotRequired[Nullable[str]]
    individual: NotRequired[Nullable[bool]]
    r"""Is this an individual or business customer"""
    project: NotRequired[Nullable[bool]]
    r"""If true, indicates this is a Project."""
    addresses: NotRequired[List[AddressTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    emails: NotRequired[List[EmailTypedDict]]
    websites: NotRequired[List[WebsiteTypedDict]]
    bank_accounts: NotRequired[List[BankAccountTypedDict]]
    notes: NotRequired[Nullable[str]]
    r"""Some notes about this customer"""
    tax_rate: NotRequired[LinkedTaxRateTypedDict]
    tax_number: NotRequired[Nullable[str]]
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    account: NotRequired[Nullable[LinkedLedgerAccountTypedDict]]
    parent: NotRequired[Nullable[LinkedParentCustomerTypedDict]]
    r"""The parent customer this entity is linked to."""
    status: NotRequired[Nullable[CustomerStatusStatus]]
    r"""Customer status"""
    payment_method: NotRequired[Nullable[str]]
    r"""Payment method used for the transaction, such as cash, credit card, bank transfer, or check"""
    channel: NotRequired[Nullable[str]]
    r"""The channel through which the transaction is processed."""
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    custom_mappings: NotRequired[Nullable[CustomMappingsTypedDict]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class Customer(BaseModel):
    id: str
    r"""A unique identifier for an object."""

    downstream_id: OptionalNullable[str] = UNSET
    r"""The third-party API ID of original entity"""

    display_id: OptionalNullable[str] = UNSET
    r"""Display ID"""

    display_name: OptionalNullable[str] = UNSET
    r"""Display name"""

    company_name: OptionalNullable[str] = UNSET
    r"""The name of the company."""

    company_id: OptionalNullable[str] = UNSET
    r"""The company or subsidiary id the transaction belongs to"""

    title: OptionalNullable[str] = UNSET
    r"""The job title of the person."""

    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the person."""

    middle_name: OptionalNullable[str] = UNSET
    r"""Middle name of the person."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the person."""

    suffix: OptionalNullable[str] = UNSET

    individual: OptionalNullable[bool] = UNSET
    r"""Is this an individual or business customer"""

    project: OptionalNullable[bool] = UNSET
    r"""If true, indicates this is a Project."""

    addresses: Optional[List[Address]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    emails: Optional[List[Email]] = None

    websites: Optional[List[Website]] = None

    bank_accounts: Optional[List[BankAccount]] = None

    notes: OptionalNullable[str] = UNSET
    r"""Some notes about this customer"""

    tax_rate: Optional[LinkedTaxRate] = None

    tax_number: OptionalNullable[str] = UNSET

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    account: OptionalNullable[LinkedLedgerAccount] = UNSET

    parent: OptionalNullable[LinkedParentCustomer] = UNSET
    r"""The parent customer this entity is linked to."""

    status: OptionalNullable[CustomerStatusStatus] = UNSET
    r"""Customer status"""

    payment_method: OptionalNullable[str] = UNSET
    r"""Payment method used for the transaction, such as cash, credit card, bank transfer, or check"""

    channel: OptionalNullable[str] = UNSET
    r"""The channel through which the transaction is processed."""

    custom_fields: Optional[List[CustomField]] = None

    custom_mappings: OptionalNullable[CustomMappings] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "downstream_id",
            "display_id",
            "display_name",
            "company_name",
            "company_id",
            "title",
            "first_name",
            "middle_name",
            "last_name",
            "suffix",
            "individual",
            "project",
            "addresses",
            "phone_numbers",
            "emails",
            "websites",
            "bank_accounts",
            "notes",
            "tax_rate",
            "tax_number",
            "currency",
            "account",
            "parent",
            "status",
            "payment_method",
            "channel",
            "custom_fields",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
            "row_version",
            "pass_through",
        ]
        nullable_fields = [
            "downstream_id",
            "display_id",
            "display_name",
            "company_name",
            "company_id",
            "title",
            "first_name",
            "middle_name",
            "last_name",
            "suffix",
            "individual",
            "project",
            "notes",
            "tax_number",
            "currency",
            "account",
            "parent",
            "status",
            "payment_method",
            "channel",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
            "row_version",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class CustomerInputTypedDict(TypedDict):
    display_id: NotRequired[Nullable[str]]
    r"""Display ID"""
    display_name: NotRequired[Nullable[str]]
    r"""Display name"""
    company_name: NotRequired[Nullable[str]]
    r"""The name of the company."""
    company_id: NotRequired[Nullable[str]]
    r"""The company or subsidiary id the transaction belongs to"""
    title: NotRequired[Nullable[str]]
    r"""The job title of the person."""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the person."""
    middle_name: NotRequired[Nullable[str]]
    r"""Middle name of the person."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the person."""
    suffix: NotRequired[Nullable[str]]
    individual: NotRequired[Nullable[bool]]
    r"""Is this an individual or business customer"""
    project: NotRequired[Nullable[bool]]
    r"""If true, indicates this is a Project."""
    addresses: NotRequired[List[AddressTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    emails: NotRequired[List[EmailTypedDict]]
    websites: NotRequired[List[WebsiteTypedDict]]
    bank_accounts: NotRequired[List[BankAccountTypedDict]]
    notes: NotRequired[Nullable[str]]
    r"""Some notes about this customer"""
    tax_rate: NotRequired[LinkedTaxRateInputTypedDict]
    tax_number: NotRequired[Nullable[str]]
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    account: NotRequired[Nullable[LinkedLedgerAccountInputTypedDict]]
    parent: NotRequired[Nullable[LinkedParentCustomerTypedDict]]
    r"""The parent customer this entity is linked to."""
    status: NotRequired[Nullable[CustomerStatusStatus]]
    r"""Customer status"""
    payment_method: NotRequired[Nullable[str]]
    r"""Payment method used for the transaction, such as cash, credit card, bank transfer, or check"""
    channel: NotRequired[Nullable[str]]
    r"""The channel through which the transaction is processed."""
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class CustomerInput(BaseModel):
    display_id: OptionalNullable[str] = UNSET
    r"""Display ID"""

    display_name: OptionalNullable[str] = UNSET
    r"""Display name"""

    company_name: OptionalNullable[str] = UNSET
    r"""The name of the company."""

    company_id: OptionalNullable[str] = UNSET
    r"""The company or subsidiary id the transaction belongs to"""

    title: OptionalNullable[str] = UNSET
    r"""The job title of the person."""

    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the person."""

    middle_name: OptionalNullable[str] = UNSET
    r"""Middle name of the person."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the person."""

    suffix: OptionalNullable[str] = UNSET

    individual: OptionalNullable[bool] = UNSET
    r"""Is this an individual or business customer"""

    project: OptionalNullable[bool] = UNSET
    r"""If true, indicates this is a Project."""

    addresses: Optional[List[Address]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    emails: Optional[List[Email]] = None

    websites: Optional[List[Website]] = None

    bank_accounts: Optional[List[BankAccount]] = None

    notes: OptionalNullable[str] = UNSET
    r"""Some notes about this customer"""

    tax_rate: Optional[LinkedTaxRateInput] = None

    tax_number: OptionalNullable[str] = UNSET

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    account: OptionalNullable[LinkedLedgerAccountInput] = UNSET

    parent: OptionalNullable[LinkedParentCustomer] = UNSET
    r"""The parent customer this entity is linked to."""

    status: OptionalNullable[CustomerStatusStatus] = UNSET
    r"""Customer status"""

    payment_method: OptionalNullable[str] = UNSET
    r"""Payment method used for the transaction, such as cash, credit card, bank transfer, or check"""

    channel: OptionalNullable[str] = UNSET
    r"""The channel through which the transaction is processed."""

    custom_fields: Optional[List[CustomField]] = None

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "display_id",
            "display_name",
            "company_name",
            "company_id",
            "title",
            "first_name",
            "middle_name",
            "last_name",
            "suffix",
            "individual",
            "project",
            "addresses",
            "phone_numbers",
            "emails",
            "websites",
            "bank_accounts",
            "notes",
            "tax_rate",
            "tax_number",
            "currency",
            "account",
            "parent",
            "status",
            "payment_method",
            "channel",
            "custom_fields",
            "row_version",
            "pass_through",
        ]
        nullable_fields = [
            "display_id",
            "display_name",
            "company_name",
            "company_id",
            "title",
            "first_name",
            "middle_name",
            "last_name",
            "suffix",
            "individual",
            "project",
            "notes",
            "tax_number",
            "currency",
            "account",
            "parent",
            "status",
            "payment_method",
            "channel",
            "row_version",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .paginationcoverage import PaginationCoverage, PaginationCoverageTypedDict
from .resourcestatus import ResourceStatus
from .supportedproperty import SupportedProperty, SupportedPropertyTypedDict
from apideck_unify.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ConnectorResourceTypedDict(TypedDict):
    id: NotRequired[str]
    r"""ID of the resource, typically a lowercased version of name."""
    name: NotRequired[str]
    r"""Name of the resource (plural)"""
    downstream_id: NotRequired[str]
    r"""ID of the resource in the Connector's API (downstream)"""
    downstream_name: NotRequired[str]
    r"""Name of the resource in the Connector's API (downstream)"""
    status: NotRequired[ResourceStatus]
    r"""Status of the resource. Resources with status live or beta are callable."""
    pagination_supported: NotRequired[bool]
    r"""Indicates if pagination (cursor and limit parameters) is supported on the list endpoint of the resource."""
    pagination: NotRequired[PaginationCoverageTypedDict]
    custom_fields_supported: NotRequired[bool]
    r"""Indicates if custom fields are supported on this resource."""
    supported_operations: NotRequired[List[str]]
    r"""List of supported operations on the resource."""
    downstream_unsupported_operations: NotRequired[List[str]]
    r"""List of operations that are not supported on the downstream."""
    supported_filters: NotRequired[List[str]]
    r"""Supported filters on the list endpoint of the resource."""
    supported_sort_by: NotRequired[List[str]]
    r"""Supported sorting properties on the list endpoint of the resource."""
    supported_fields: NotRequired[List[SupportedPropertyTypedDict]]
    r"""Supported fields on the detail endpoint."""
    supported_list_fields: NotRequired[List[SupportedPropertyTypedDict]]
    r"""Supported fields on the list endpoint."""


class ConnectorResource(BaseModel):
    id: Optional[str] = None
    r"""ID of the resource, typically a lowercased version of name."""

    name: Optional[str] = None
    r"""Name of the resource (plural)"""

    downstream_id: Optional[str] = None
    r"""ID of the resource in the Connector's API (downstream)"""

    downstream_name: Optional[str] = None
    r"""Name of the resource in the Connector's API (downstream)"""

    status: Optional[ResourceStatus] = None
    r"""Status of the resource. Resources with status live or beta are callable."""

    pagination_supported: Optional[bool] = None
    r"""Indicates if pagination (cursor and limit parameters) is supported on the list endpoint of the resource."""

    pagination: Optional[PaginationCoverage] = None

    custom_fields_supported: Optional[bool] = None
    r"""Indicates if custom fields are supported on this resource."""

    supported_operations: Optional[List[str]] = None
    r"""List of supported operations on the resource."""

    downstream_unsupported_operations: Optional[List[str]] = None
    r"""List of operations that are not supported on the downstream."""

    supported_filters: Optional[List[str]] = None
    r"""Supported filters on the list endpoint of the resource."""

    supported_sort_by: Optional[List[str]] = None
    r"""Supported sorting properties on the list endpoint of the resource."""

    supported_fields: Optional[List[SupportedProperty]] = None
    r"""Supported fields on the detail endpoint."""

    supported_list_fields: Optional[List[SupportedProperty]] = None
    r"""Supported fields on the list endpoint."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class WebhookSubscriptionTypedDict(TypedDict):
    downstream_id: NotRequired[str]
    r"""The ID of the downstream service"""
    unify_event_types: NotRequired[List[str]]
    r"""The list of Unify Events this connection is subscribed to"""
    downstream_event_types: NotRequired[List[str]]
    r"""The list of downstream Events this connection is subscribed to"""
    execute_url: NotRequired[str]
    r"""The URL the downstream is sending to when the event is triggered"""
    created_at: NotRequired[str]
    r"""The date and time the webhook subscription was created downstream"""


class WebhookSubscription(BaseModel):
    downstream_id: Optional[str] = None
    r"""The ID of the downstream service"""

    unify_event_types: Optional[List[str]] = None
    r"""The list of Unify Events this connection is subscribed to"""

    downstream_event_types: Optional[List[str]] = None
    r"""The list of downstream Events this connection is subscribed to"""

    execute_url: Optional[str] = None
    r"""The URL the downstream is sending to when the event is triggered"""

    created_at: Optional[str] = None
    r"""The date and time the webhook subscription was created downstream"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .apistatus import APIStatus
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ApisFilterTypedDict(TypedDict):
    status: NotRequired[APIStatus]
    r"""Status of the API. APIs with status live or beta are callable."""


class ApisFilter(BaseModel):
    status: Annotated[Optional[APIStatus], FieldMetadata(query=True)] = None
    r"""Status of the API. APIs with status live or beta are callable."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .formfieldoptiongroup import FormFieldOptionGroup, FormFieldOptionGroupTypedDict
from .simpleformfieldoption import SimpleFormFieldOption, SimpleFormFieldOptionTypedDict
from apideck_unify.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


FormFieldOptionTypedDict = TypeAliasType(
    "FormFieldOptionTypedDict",
    Union[SimpleFormFieldOptionTypedDict, FormFieldOptionGroupTypedDict],
)


FormFieldOption = Annotated[
    Union[
        Annotated[SimpleFormFieldOption, Tag("simple")],
        Annotated[FormFieldOptionGroup, Tag("group")],
    ],
    Discriminator(lambda m: get_discriminator(m, "option_type", "option_type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefit import Benefit, BenefitTypedDict
from .deduction import Deduction, DeductionTypedDict
from .tax import Tax, TaxTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List
from typing_extensions import NotRequired, TypedDict


class CompensationTypedDict(TypedDict):
    employee_id: Nullable[str]
    r"""A unique identifier for an object."""
    net_pay: NotRequired[Nullable[float]]
    r"""The employee's net pay. Only available when payroll has been processed"""
    gross_pay: NotRequired[Nullable[float]]
    r"""The employee's gross pay. Only available when payroll has been processed"""
    taxes: NotRequired[Nullable[List[TaxTypedDict]]]
    r"""An array of employer and employee taxes for the pay period."""
    deductions: NotRequired[Nullable[List[DeductionTypedDict]]]
    r"""An array of employee deductions for the pay period."""
    benefits: NotRequired[Nullable[List[BenefitTypedDict]]]
    r"""An array of employee benefits for the pay period."""


class Compensation(BaseModel):
    employee_id: Nullable[str]
    r"""A unique identifier for an object."""

    net_pay: OptionalNullable[float] = UNSET
    r"""The employee's net pay. Only available when payroll has been processed"""

    gross_pay: OptionalNullable[float] = UNSET
    r"""The employee's gross pay. Only available when payroll has been processed"""

    taxes: OptionalNullable[List[Tax]] = UNSET
    r"""An array of employer and employee taxes for the pay period."""

    deductions: OptionalNullable[List[Deduction]] = UNSET
    r"""An array of employee deductions for the pay period."""

    benefits: OptionalNullable[List[Benefit]] = UNSET
    r"""An array of employee benefits for the pay period."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["net_pay", "gross_pay", "taxes", "deductions", "benefits"]
        nullable_fields = [
            "employee_id",
            "net_pay",
            "gross_pay",
            "taxes",
            "deductions",
            "benefits",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from enum import Enum
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class EventSource(str, Enum):
    r"""Unify event source"""

    NATIVE = "native"
    VIRTUAL = "virtual"


class ConnectorEventTypedDict(TypedDict):
    r"""Unify event that is supported on the connector. Events are delivered via Webhooks."""

    event_type: NotRequired[str]
    r"""Unify event type"""
    event_source: NotRequired[EventSource]
    r"""Unify event source"""
    downstream_event_type: NotRequired[str]
    r"""Downstream event type"""
    resources: NotRequired[List[str]]
    entity_type: NotRequired[str]
    r"""Unify entity type"""


class ConnectorEvent(BaseModel):
    r"""Unify event that is supported on the connector. Events are delivered via Webhooks."""

    event_type: Optional[str] = None
    r"""Unify event type"""

    event_source: Optional[EventSource] = None
    r"""Unify event source"""

    downstream_event_type: Optional[str] = None
    r"""Downstream event type"""

    resources: Optional[List[str]] = None

    entity_type: Optional[str] = None
    r"""Unify entity type"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CreateCallbackStateTypedDict(TypedDict):
    redirect_uri: NotRequired[str]
    r"""The redirect URI to be used after the connection is created."""


class CreateCallbackState(BaseModel):
    redirect_uri: Optional[str] = None
    r"""The redirect URI to be used after the connection is created."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .currency import Currency
from .custommappings import CustomMappings, CustomMappingsTypedDict
from .email import Email, EmailTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from .taxrate import TaxRate, TaxRateTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import date, datetime
from enum import Enum
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class CompanyStatus(str, Enum):
    r"""Based on the status some functionality is enabled or disabled."""

    ACTIVE = "active"
    INACTIVE = "inactive"


class TheStartMonthOfFiscalYear(str, Enum):
    r"""The start month of fiscal year."""

    JANUARY = "January"
    FEBRUARY = "February"
    MARCH = "March"
    APRIL = "April"
    MAY = "May"
    JUNE = "June"
    JULY = "July"
    AUGUST = "August"
    SEPTEMBER = "September"
    OCTOBER = "October"
    NOVEMBER = "November"
    DECEMBER = "December"


class CompanyInfoTypedDict(TypedDict):
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    company_name: NotRequired[Nullable[str]]
    r"""The name of the company."""
    status: NotRequired[CompanyStatus]
    r"""Based on the status some functionality is enabled or disabled."""
    legal_name: NotRequired[str]
    r"""The legal name of the company"""
    country: NotRequired[Nullable[str]]
    r"""country code according to ISO 3166-1 alpha-2."""
    sales_tax_number: NotRequired[Nullable[str]]
    automated_sales_tax: NotRequired[bool]
    r"""Whether sales tax is calculated automatically for the company"""
    sales_tax_enabled: NotRequired[bool]
    r"""Whether sales tax is enabled for the company"""
    default_sales_tax: NotRequired[TaxRateTypedDict]
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    language: NotRequired[Nullable[str]]
    r"""language code according to ISO 639-1. For the United States - EN"""
    fiscal_year_start_month: NotRequired[TheStartMonthOfFiscalYear]
    r"""The start month of fiscal year."""
    company_start_date: NotRequired[date]
    r"""Date when company file was created"""
    addresses: NotRequired[List[AddressTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    emails: NotRequired[List[EmailTypedDict]]
    custom_mappings: NotRequired[Nullable[CustomMappingsTypedDict]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""


class CompanyInfo(BaseModel):
    id: Optional[str] = None
    r"""A unique identifier for an object."""

    company_name: OptionalNullable[str] = UNSET
    r"""The name of the company."""

    status: Optional[CompanyStatus] = None
    r"""Based on the status some functionality is enabled or disabled."""

    legal_name: Optional[str] = None
    r"""The legal name of the company"""

    country: OptionalNullable[str] = UNSET
    r"""country code according to ISO 3166-1 alpha-2."""

    sales_tax_number: OptionalNullable[str] = UNSET

    automated_sales_tax: Optional[bool] = None
    r"""Whether sales tax is calculated automatically for the company"""

    sales_tax_enabled: Optional[bool] = None
    r"""Whether sales tax is enabled for the company"""

    default_sales_tax: Optional[TaxRate] = None

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    language: OptionalNullable[str] = UNSET
    r"""language code according to ISO 639-1. For the United States - EN"""

    fiscal_year_start_month: Optional[TheStartMonthOfFiscalYear] = None
    r"""The start month of fiscal year."""

    company_start_date: Optional[date] = None
    r"""Date when company file was created"""

    addresses: Optional[List[Address]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    emails: Optional[List[Email]] = None

    custom_mappings: OptionalNullable[CustomMappings] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "company_name",
            "status",
            "legal_name",
            "country",
            "sales_tax_number",
            "automated_sales_tax",
            "sales_tax_enabled",
            "default_sales_tax",
            "currency",
            "language",
            "fiscal_year_start_month",
            "company_start_date",
            "addresses",
            "phone_numbers",
            "emails",
            "custom_mappings",
            "row_version",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        nullable_fields = [
            "company_name",
            "country",
            "sales_tax_number",
            "currency",
            "language",
            "custom_mappings",
            "row_version",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .bankaccount import BankAccount, BankAccountTypedDict
from .currency import Currency
from .customfield import CustomField, CustomFieldTypedDict
from .custommappings import CustomMappings, CustomMappingsTypedDict
from .invoicelineitem import (
    InvoiceLineItem,
    InvoiceLineItemInput,
    InvoiceLineItemInputTypedDict,
    InvoiceLineItemTypedDict,
)
from .linkedledgeraccount import LinkedLedgerAccount, LinkedLedgerAccountTypedDict
from .linkedledgeraccount_input import (
    LinkedLedgerAccountInput,
    LinkedLedgerAccountInputTypedDict,
)
from .linkedsupplier import LinkedSupplier, LinkedSupplierTypedDict
from .linkedsupplier_input import LinkedSupplierInput, LinkedSupplierInputTypedDict
from .linkedtrackingcategory import (
    LinkedTrackingCategory,
    LinkedTrackingCategoryTypedDict,
)
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import date, datetime
from enum import Enum
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class PurchaseOrderStatus(str, Enum):
    DRAFT = "draft"
    OPEN = "open"
    CLOSED = "closed"
    DELETED = "deleted"
    BILLED = "billed"
    OTHER = "other"


class PurchaseOrderTypedDict(TypedDict):
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    downstream_id: NotRequired[Nullable[str]]
    r"""The third-party API ID of original entity"""
    po_number: NotRequired[Nullable[str]]
    r"""A PO Number uniquely identifies a purchase order and is generally defined by the buyer."""
    reference: NotRequired[Nullable[str]]
    r"""Optional purchase order reference."""
    supplier: NotRequired[Nullable[LinkedSupplierTypedDict]]
    r"""The supplier this entity is linked to."""
    company_id: NotRequired[Nullable[str]]
    r"""The company or subsidiary id the transaction belongs to"""
    status: NotRequired[Nullable[PurchaseOrderStatus]]
    issued_date: NotRequired[Nullable[date]]
    r"""Date purchase order was issued - YYYY-MM-DD."""
    delivery_date: NotRequired[Nullable[date]]
    r"""The date on which the purchase order is to be delivered - YYYY-MM-DD."""
    expected_arrival_date: NotRequired[Nullable[date]]
    r"""The date on which the order is expected to arrive - YYYY-MM-DD."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    currency_rate: NotRequired[Nullable[float]]
    r"""Currency Exchange Rate at the time entity was recorded/generated."""
    sub_total: NotRequired[Nullable[float]]
    r"""Sub-total amount, normally before tax."""
    total_tax: NotRequired[Nullable[float]]
    r"""Total tax amount applied to this invoice."""
    total: NotRequired[Nullable[float]]
    r"""Total amount of invoice, including tax."""
    tax_inclusive: NotRequired[Nullable[bool]]
    r"""Amounts are including tax"""
    line_items: NotRequired[List[InvoiceLineItemTypedDict]]
    shipping_address: NotRequired[AddressTypedDict]
    ledger_account: NotRequired[Nullable[LinkedLedgerAccountTypedDict]]
    template_id: NotRequired[Nullable[str]]
    r"""Optional purchase order template"""
    discount_percentage: NotRequired[Nullable[float]]
    r"""Discount percentage applied to this transaction."""
    bank_account: NotRequired[BankAccountTypedDict]
    accounting_by_row: NotRequired[Nullable[bool]]
    r"""Indicates if accounting by row is used (true) or not (false). Accounting by row means that a separate ledger transaction is created for each row."""
    due_date: NotRequired[Nullable[date]]
    r"""The due date is the date on which a payment is scheduled to be received - YYYY-MM-DD."""
    payment_method: NotRequired[Nullable[str]]
    r"""Payment method used for the transaction, such as cash, credit card, bank transfer, or check"""
    tax_code: NotRequired[Nullable[str]]
    r"""Applicable tax id/code override if tax is not supplied on a line item basis."""
    channel: NotRequired[Nullable[str]]
    r"""The channel through which the transaction is processed."""
    memo: NotRequired[Nullable[str]]
    r"""Message for the supplier. This text appears on the Purchase Order."""
    tracking_categories: NotRequired[
        Nullable[List[Nullable[LinkedTrackingCategoryTypedDict]]]
    ]
    r"""A list of linked tracking categories."""
    custom_mappings: NotRequired[Nullable[CustomMappingsTypedDict]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class PurchaseOrder(BaseModel):
    id: Optional[str] = None
    r"""A unique identifier for an object."""

    downstream_id: OptionalNullable[str] = UNSET
    r"""The third-party API ID of original entity"""

    po_number: OptionalNullable[str] = UNSET
    r"""A PO Number uniquely identifies a purchase order and is generally defined by the buyer."""

    reference: OptionalNullable[str] = UNSET
    r"""Optional purchase order reference."""

    supplier: OptionalNullable[LinkedSupplier] = UNSET
    r"""The supplier this entity is linked to."""

    company_id: OptionalNullable[str] = UNSET
    r"""The company or subsidiary id the transaction belongs to"""

    status: OptionalNullable[PurchaseOrderStatus] = UNSET

    issued_date: OptionalNullable[date] = UNSET
    r"""Date purchase order was issued - YYYY-MM-DD."""

    delivery_date: OptionalNullable[date] = UNSET
    r"""The date on which the purchase order is to be delivered - YYYY-MM-DD."""

    expected_arrival_date: OptionalNullable[date] = UNSET
    r"""The date on which the order is expected to arrive - YYYY-MM-DD."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    currency_rate: OptionalNullable[float] = UNSET
    r"""Currency Exchange Rate at the time entity was recorded/generated."""

    sub_total: OptionalNullable[float] = UNSET
    r"""Sub-total amount, normally before tax."""

    total_tax: OptionalNullable[float] = UNSET
    r"""Total tax amount applied to this invoice."""

    total: OptionalNullable[float] = UNSET
    r"""Total amount of invoice, including tax."""

    tax_inclusive: OptionalNullable[bool] = UNSET
    r"""Amounts are including tax"""

    line_items: Optional[List[InvoiceLineItem]] = None

    shipping_address: Optional[Address] = None

    ledger_account: OptionalNullable[LinkedLedgerAccount] = UNSET

    template_id: OptionalNullable[str] = UNSET
    r"""Optional purchase order template"""

    discount_percentage: OptionalNullable[float] = UNSET
    r"""Discount percentage applied to this transaction."""

    bank_account: Optional[BankAccount] = None

    accounting_by_row: OptionalNullable[bool] = UNSET
    r"""Indicates if accounting by row is used (true) or not (false). Accounting by row means that a separate ledger transaction is created for each row."""

    due_date: OptionalNullable[date] = UNSET
    r"""The due date is the date on which a payment is scheduled to be received - YYYY-MM-DD."""

    payment_method: OptionalNullable[str] = UNSET
    r"""Payment method used for the transaction, such as cash, credit card, bank transfer, or check"""

    tax_code: OptionalNullable[str] = UNSET
    r"""Applicable tax id/code override if tax is not supplied on a line item basis."""

    channel: OptionalNullable[str] = UNSET
    r"""The channel through which the transaction is processed."""

    memo: OptionalNullable[str] = UNSET
    r"""Message for the supplier. This text appears on the Purchase Order."""

    tracking_categories: OptionalNullable[List[Nullable[LinkedTrackingCategory]]] = (
        UNSET
    )
    r"""A list of linked tracking categories."""

    custom_mappings: OptionalNullable[CustomMappings] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    custom_fields: Optional[List[CustomField]] = None

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "downstream_id",
            "po_number",
            "reference",
            "supplier",
            "company_id",
            "status",
            "issued_date",
            "delivery_date",
            "expected_arrival_date",
            "currency",
            "currency_rate",
            "sub_total",
            "total_tax",
            "total",
            "tax_inclusive",
            "line_items",
            "shipping_address",
            "ledger_account",
            "template_id",
            "discount_percentage",
            "bank_account",
            "accounting_by_row",
            "due_date",
            "payment_method",
            "tax_code",
            "channel",
            "memo",
            "tracking_categories",
            "custom_mappings",
            "custom_fields",
            "row_version",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
            "pass_through",
        ]
        nullable_fields = [
            "downstream_id",
            "po_number",
            "reference",
            "supplier",
            "company_id",
            "status",
            "issued_date",
            "delivery_date",
            "expected_arrival_date",
            "currency",
            "currency_rate",
            "sub_total",
            "total_tax",
            "total",
            "tax_inclusive",
            "ledger_account",
            "template_id",
            "discount_percentage",
            "accounting_by_row",
            "due_date",
            "payment_method",
            "tax_code",
            "channel",
            "memo",
            "tracking_categories",
            "custom_mappings",
            "row_version",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class PurchaseOrderInputTypedDict(TypedDict):
    po_number: NotRequired[Nullable[str]]
    r"""A PO Number uniquely identifies a purchase order and is generally defined by the buyer."""
    reference: NotRequired[Nullable[str]]
    r"""Optional purchase order reference."""
    supplier: NotRequired[Nullable[LinkedSupplierInputTypedDict]]
    r"""The supplier this entity is linked to."""
    company_id: NotRequired[Nullable[str]]
    r"""The company or subsidiary id the transaction belongs to"""
    status: NotRequired[Nullable[PurchaseOrderStatus]]
    issued_date: NotRequired[Nullable[date]]
    r"""Date purchase order was issued - YYYY-MM-DD."""
    delivery_date: NotRequired[Nullable[date]]
    r"""The date on which the purchase order is to be delivered - YYYY-MM-DD."""
    expected_arrival_date: NotRequired[Nullable[date]]
    r"""The date on which the order is expected to arrive - YYYY-MM-DD."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    currency_rate: NotRequired[Nullable[float]]
    r"""Currency Exchange Rate at the time entity was recorded/generated."""
    sub_total: NotRequired[Nullable[float]]
    r"""Sub-total amount, normally before tax."""
    total_tax: NotRequired[Nullable[float]]
    r"""Total tax amount applied to this invoice."""
    total: NotRequired[Nullable[float]]
    r"""Total amount of invoice, including tax."""
    tax_inclusive: NotRequired[Nullable[bool]]
    r"""Amounts are including tax"""
    line_items: NotRequired[List[InvoiceLineItemInputTypedDict]]
    shipping_address: NotRequired[AddressTypedDict]
    ledger_account: NotRequired[Nullable[LinkedLedgerAccountInputTypedDict]]
    template_id: NotRequired[Nullable[str]]
    r"""Optional purchase order template"""
    discount_percentage: NotRequired[Nullable[float]]
    r"""Discount percentage applied to this transaction."""
    bank_account: NotRequired[BankAccountTypedDict]
    accounting_by_row: NotRequired[Nullable[bool]]
    r"""Indicates if accounting by row is used (true) or not (false). Accounting by row means that a separate ledger transaction is created for each row."""
    due_date: NotRequired[Nullable[date]]
    r"""The due date is the date on which a payment is scheduled to be received - YYYY-MM-DD."""
    payment_method: NotRequired[Nullable[str]]
    r"""Payment method used for the transaction, such as cash, credit card, bank transfer, or check"""
    tax_code: NotRequired[Nullable[str]]
    r"""Applicable tax id/code override if tax is not supplied on a line item basis."""
    channel: NotRequired[Nullable[str]]
    r"""The channel through which the transaction is processed."""
    memo: NotRequired[Nullable[str]]
    r"""Message for the supplier. This text appears on the Purchase Order."""
    tracking_categories: NotRequired[
        Nullable[List[Nullable[LinkedTrackingCategoryTypedDict]]]
    ]
    r"""A list of linked tracking categories."""
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class PurchaseOrderInput(BaseModel):
    po_number: OptionalNullable[str] = UNSET
    r"""A PO Number uniquely identifies a purchase order and is generally defined by the buyer."""

    reference: OptionalNullable[str] = UNSET
    r"""Optional purchase order reference."""

    supplier: OptionalNullable[LinkedSupplierInput] = UNSET
    r"""The supplier this entity is linked to."""

    company_id: OptionalNullable[str] = UNSET
    r"""The company or subsidiary id the transaction belongs to"""

    status: OptionalNullable[PurchaseOrderStatus] = UNSET

    issued_date: OptionalNullable[date] = UNSET
    r"""Date purchase order was issued - YYYY-MM-DD."""

    delivery_date: OptionalNullable[date] = UNSET
    r"""The date on which the purchase order is to be delivered - YYYY-MM-DD."""

    expected_arrival_date: OptionalNullable[date] = UNSET
    r"""The date on which the order is expected to arrive - YYYY-MM-DD."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    currency_rate: OptionalNullable[float] = UNSET
    r"""Currency Exchange Rate at the time entity was recorded/generated."""

    sub_total: OptionalNullable[float] = UNSET
    r"""Sub-total amount, normally before tax."""

    total_tax: OptionalNullable[float] = UNSET
    r"""Total tax amount applied to this invoice."""

    total: OptionalNullable[float] = UNSET
    r"""Total amount of invoice, including tax."""

    tax_inclusive: OptionalNullable[bool] = UNSET
    r"""Amounts are including tax"""

    line_items: Optional[List[InvoiceLineItemInput]] = None

    shipping_address: Optional[Address] = None

    ledger_account: OptionalNullable[LinkedLedgerAccountInput] = UNSET

    template_id: OptionalNullable[str] = UNSET
    r"""Optional purchase order template"""

    discount_percentage: OptionalNullable[float] = UNSET
    r"""Discount percentage applied to this transaction."""

    bank_account: Optional[BankAccount] = None

    accounting_by_row: OptionalNullable[bool] = UNSET
    r"""Indicates if accounting by row is used (true) or not (false). Accounting by row means that a separate ledger transaction is created for each row."""

    due_date: OptionalNullable[date] = UNSET
    r"""The due date is the date on which a payment is scheduled to be received - YYYY-MM-DD."""

    payment_method: OptionalNullable[str] = UNSET
    r"""Payment method used for the transaction, such as cash, credit card, bank transfer, or check"""

    tax_code: OptionalNullable[str] = UNSET
    r"""Applicable tax id/code override if tax is not supplied on a line item basis."""

    channel: OptionalNullable[str] = UNSET
    r"""The channel through which the transaction is processed."""

    memo: OptionalNullable[str] = UNSET
    r"""Message for the supplier. This text appears on the Purchase Order."""

    tracking_categories: OptionalNullable[List[Nullable[LinkedTrackingCategory]]] = (
        UNSET
    )
    r"""A list of linked tracking categories."""

    custom_fields: Optional[List[CustomField]] = None

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "po_number",
            "reference",
            "supplier",
            "company_id",
            "status",
            "issued_date",
            "delivery_date",
            "expected_arrival_date",
            "currency",
            "currency_rate",
            "sub_total",
            "total_tax",
            "total",
            "tax_inclusive",
            "line_items",
            "shipping_address",
            "ledger_account",
            "template_id",
            "discount_percentage",
            "bank_account",
            "accounting_by_row",
            "due_date",
            "payment_method",
            "tax_code",
            "channel",
            "memo",
            "tracking_categories",
            "custom_fields",
            "row_version",
            "pass_through",
        ]
        nullable_fields = [
            "po_number",
            "reference",
            "supplier",
            "company_id",
            "status",
            "issued_date",
            "delivery_date",
            "expected_arrival_date",
            "currency",
            "currency_rate",
            "sub_total",
            "total_tax",
            "total",
            "tax_inclusive",
            "ledger_account",
            "template_id",
            "discount_percentage",
            "accounting_by_row",
            "due_date",
            "payment_method",
            "tax_code",
            "channel",
            "memo",
            "tracking_categories",
            "row_version",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from apideck_unify.apideck_customers import ApideckCustomers
from apideck_unify.orders import Orders
from apideck_unify.products import Products
from apideck_unify.stores import Stores


class Ecommerce(BaseSDK):
    orders: Orders
    products: Products
    customers: ApideckCustomers
    stores: Stores

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.orders = Orders(self.sdk_configuration)
        self.products = Products(self.sdk_configuration)
        self.customers = ApideckCustomers(self.sdk_configuration)
        self.stores = Stores(self.sdk_configuration)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .bankaccount import BankAccount, BankAccountTypedDict
from .customfield import CustomField, CustomFieldTypedDict
from .custommappings import CustomMappings, CustomMappingsTypedDict
from .email import Email, EmailTypedDict
from .employeecompensation import (
    EmployeeCompensation,
    EmployeeCompensationInput,
    EmployeeCompensationInputTypedDict,
    EmployeeCompensationTypedDict,
)
from .employeejob import (
    EmployeeJob,
    EmployeeJobInput,
    EmployeeJobInputTypedDict,
    EmployeeJobTypedDict,
)
from .employmentstatus import EmploymentStatus
from .gender import Gender
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from .person import Person, PersonTypedDict
from .person_input import PersonInput, PersonInputTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from .sociallink import SocialLink, SocialLinkTypedDict
from .team import Team, TeamTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import date, datetime
from enum import Enum
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LeavingReason(str, Enum):
    r"""The reason because the employment ended."""

    DISMISSED = "dismissed"
    RESIGNED = "resigned"
    REDUNDANCY = "redundancy"
    OTHER = "other"


class EmploymentType(str, Enum):
    r"""The type of employment relationship the employee has with the organization."""

    CONTRACTOR = "contractor"
    EMPLOYEE = "employee"
    FREELANCE = "freelance"
    TEMP = "temp"
    INTERNSHIP = "internship"
    OTHER = "other"


class EmploymentSubType(str, Enum):
    r"""The work schedule of the employee."""

    FULL_TIME = "full_time"
    PART_TIME = "part_time"
    HOURLY = "hourly"
    OTHER = "other"
    NOT_SPECIFIED = "not_specified"


class EmploymentRoleTypedDict(TypedDict):
    type: NotRequired[Nullable[EmploymentType]]
    r"""The type of employment relationship the employee has with the organization."""
    sub_type: NotRequired[Nullable[EmploymentSubType]]
    r"""The work schedule of the employee."""


class EmploymentRole(BaseModel):
    type: OptionalNullable[EmploymentType] = UNSET
    r"""The type of employment relationship the employee has with the organization."""

    sub_type: OptionalNullable[EmploymentSubType] = UNSET
    r"""The work schedule of the employee."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["type", "sub_type"]
        nullable_fields = ["type", "sub_type"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ManagerTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for an object."""
    name: NotRequired[Nullable[str]]
    r"""The name of the manager, often a combination of their first and last names."""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the person."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the person."""
    email: NotRequired[Nullable[str]]
    r"""The email address of the manager."""
    employment_status: NotRequired[Nullable[EmploymentStatus]]
    r"""The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status."""


class Manager(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for an object."""

    name: OptionalNullable[str] = UNSET
    r"""The name of the manager, often a combination of their first and last names."""

    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the person."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the person."""

    email: OptionalNullable[str] = UNSET
    r"""The email address of the manager."""

    employment_status: OptionalNullable[EmploymentStatus] = UNSET
    r"""The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "name",
            "first_name",
            "last_name",
            "email",
            "employment_status",
        ]
        nullable_fields = [
            "id",
            "name",
            "first_name",
            "last_name",
            "email",
            "employment_status",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ProbationPeriodTypedDict(TypedDict):
    start_date: NotRequired[Nullable[date]]
    r"""The date that the employee started their probation period at the company."""
    end_date: NotRequired[Nullable[date]]
    r"""The date that the employee ended their probation period at the company."""


class ProbationPeriod(BaseModel):
    start_date: OptionalNullable[date] = UNSET
    r"""The date that the employee started their probation period at the company."""

    end_date: OptionalNullable[date] = UNSET
    r"""The date that the employee ended their probation period at the company."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["start_date", "end_date"]
        nullable_fields = ["start_date", "end_date"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EmployeeTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for an object."""
    downstream_id: NotRequired[Nullable[str]]
    r"""The third-party API ID of original entity"""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the person."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the person."""
    middle_name: NotRequired[Nullable[str]]
    r"""Middle name of the person."""
    display_name: NotRequired[Nullable[str]]
    r"""The name used to display the employee, often a combination of their first and last names."""
    preferred_name: NotRequired[Nullable[str]]
    r"""The name the employee prefers to be addressed by, which may be different from their legal name."""
    initials: NotRequired[Nullable[str]]
    r"""The initials of the person, usually derived from their first, middle, and last names."""
    salutation: NotRequired[Nullable[str]]
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""
    title: NotRequired[Nullable[str]]
    r"""The job title of the person."""
    marital_status: NotRequired[Nullable[str]]
    r"""The marital status of the employee."""
    partner: NotRequired[PersonTypedDict]
    division: NotRequired[Nullable[str]]
    r"""The division the person is currently in. Usually a collection of departments or teams or regions."""
    division_id: NotRequired[Nullable[str]]
    r"""Unique identifier of the division this employee belongs to."""
    department: NotRequired[Nullable[str]]
    r"""The department the person is currently in. [Deprecated](https://developers.apideck.com/changelog) in favor of the dedicated department_id and department_name field."""
    department_id: NotRequired[Nullable[str]]
    r"""Unique identifier of the department ID this employee belongs to."""
    department_name: NotRequired[Nullable[str]]
    r"""Name of the department this employee belongs to."""
    team: NotRequired[Nullable[TeamTypedDict]]
    r"""The team the person is currently in."""
    company_id: NotRequired[Nullable[str]]
    r"""The unique identifier of the company."""
    company_name: NotRequired[Nullable[str]]
    r"""The name of the company."""
    employment_start_date: NotRequired[Nullable[str]]
    r"""A Start Date is the date that the employee started working at the company"""
    employment_end_date: NotRequired[Nullable[str]]
    r"""An End Date is the date that the employee ended working at the company"""
    leaving_reason: NotRequired[Nullable[LeavingReason]]
    r"""The reason because the employment ended."""
    employee_number: NotRequired[Nullable[str]]
    r"""An Employee Number, Employee ID or Employee Code, is a unique number that has been assigned to each individual staff member within a company."""
    employment_status: NotRequired[Nullable[EmploymentStatus]]
    r"""The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status."""
    employment_role: NotRequired[EmploymentRoleTypedDict]
    ethnicity: NotRequired[Nullable[str]]
    r"""The ethnicity of the employee"""
    manager: NotRequired[ManagerTypedDict]
    direct_reports: NotRequired[Nullable[List[str]]]
    r"""Direct reports is an array of ids that reflect the individuals in an organizational hierarchy who are directly supervised by this specific employee."""
    social_security_number: NotRequired[Nullable[str]]
    r"""A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions."""
    birthday: NotRequired[Nullable[date]]
    r"""The date of birth of the person."""
    deceased_on: NotRequired[Nullable[date]]
    r"""The date the person deceased."""
    country_of_birth: NotRequired[Nullable[str]]
    r"""Country code according to ISO 3166-1 alpha-2."""
    description: NotRequired[Nullable[str]]
    r"""A description of the object."""
    gender: NotRequired[Nullable[Gender]]
    r"""The gender represents the gender identity of a person."""
    pronouns: NotRequired[Nullable[str]]
    r"""The preferred pronouns of the person."""
    preferred_language: NotRequired[Nullable[str]]
    r"""language code according to ISO 639-1. For the United States - EN"""
    languages: NotRequired[List[Nullable[str]]]
    nationalities: NotRequired[List[Nullable[str]]]
    photo_url: NotRequired[Nullable[str]]
    r"""The URL of the photo of a person."""
    timezone: NotRequired[Nullable[str]]
    r"""The time zone related to the resource. The value is a string containing a standard time zone identifier, e.g. Europe/London."""
    source: NotRequired[Nullable[str]]
    r"""When the employee is imported as a new hire, this field indicates what system (e.g. the name of the ATS) this employee was imported from."""
    source_id: NotRequired[Nullable[str]]
    r"""Unique identifier of the employee in the system this employee was imported from (e.g. the ID in the ATS)."""
    record_url: NotRequired[Nullable[str]]
    jobs: NotRequired[Nullable[List[EmployeeJobTypedDict]]]
    compensations: NotRequired[Nullable[List[EmployeeCompensationTypedDict]]]
    works_remote: NotRequired[Nullable[bool]]
    r"""Indicates if the employee works from a remote location."""
    addresses: NotRequired[List[AddressTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    emails: NotRequired[List[EmailTypedDict]]
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    social_links: NotRequired[List[SocialLinkTypedDict]]
    bank_accounts: NotRequired[List[BankAccountTypedDict]]
    tax_code: NotRequired[Nullable[str]]
    tax_id: NotRequired[Nullable[str]]
    dietary_preference: NotRequired[Nullable[str]]
    r"""Indicate the employee's dietary preference."""
    food_allergies: NotRequired[Nullable[List[str]]]
    r"""Indicate the employee's food allergies."""
    probation_period: NotRequired[ProbationPeriodTypedDict]
    tags: NotRequired[Nullable[List[str]]]
    custom_mappings: NotRequired[Nullable[CustomMappingsTypedDict]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    deleted: NotRequired[Nullable[bool]]
    r"""Flag to indicate if the object is deleted."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class Employee(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for an object."""

    downstream_id: OptionalNullable[str] = UNSET
    r"""The third-party API ID of original entity"""

    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the person."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the person."""

    middle_name: OptionalNullable[str] = UNSET
    r"""Middle name of the person."""

    display_name: OptionalNullable[str] = UNSET
    r"""The name used to display the employee, often a combination of their first and last names."""

    preferred_name: OptionalNullable[str] = UNSET
    r"""The name the employee prefers to be addressed by, which may be different from their legal name."""

    initials: OptionalNullable[str] = UNSET
    r"""The initials of the person, usually derived from their first, middle, and last names."""

    salutation: OptionalNullable[str] = UNSET
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""

    title: OptionalNullable[str] = UNSET
    r"""The job title of the person."""

    marital_status: OptionalNullable[str] = UNSET
    r"""The marital status of the employee."""

    partner: Optional[Person] = None

    division: OptionalNullable[str] = UNSET
    r"""The division the person is currently in. Usually a collection of departments or teams or regions."""

    division_id: OptionalNullable[str] = UNSET
    r"""Unique identifier of the division this employee belongs to."""

    department: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""The department the person is currently in. [Deprecated](https://developers.apideck.com/changelog) in favor of the dedicated department_id and department_name field."""

    department_id: OptionalNullable[str] = UNSET
    r"""Unique identifier of the department ID this employee belongs to."""

    department_name: OptionalNullable[str] = UNSET
    r"""Name of the department this employee belongs to."""

    team: OptionalNullable[Team] = UNSET
    r"""The team the person is currently in."""

    company_id: OptionalNullable[str] = UNSET
    r"""The unique identifier of the company."""

    company_name: OptionalNullable[str] = UNSET
    r"""The name of the company."""

    employment_start_date: OptionalNullable[str] = UNSET
    r"""A Start Date is the date that the employee started working at the company"""

    employment_end_date: OptionalNullable[str] = UNSET
    r"""An End Date is the date that the employee ended working at the company"""

    leaving_reason: OptionalNullable[LeavingReason] = UNSET
    r"""The reason because the employment ended."""

    employee_number: OptionalNullable[str] = UNSET
    r"""An Employee Number, Employee ID or Employee Code, is a unique number that has been assigned to each individual staff member within a company."""

    employment_status: OptionalNullable[EmploymentStatus] = UNSET
    r"""The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status."""

    employment_role: Optional[EmploymentRole] = None

    ethnicity: OptionalNullable[str] = UNSET
    r"""The ethnicity of the employee"""

    manager: Optional[Manager] = None

    direct_reports: OptionalNullable[List[str]] = UNSET
    r"""Direct reports is an array of ids that reflect the individuals in an organizational hierarchy who are directly supervised by this specific employee."""

    social_security_number: OptionalNullable[str] = UNSET
    r"""A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions."""

    birthday: OptionalNullable[date] = UNSET
    r"""The date of birth of the person."""

    deceased_on: OptionalNullable[date] = UNSET
    r"""The date the person deceased."""

    country_of_birth: OptionalNullable[str] = UNSET
    r"""Country code according to ISO 3166-1 alpha-2."""

    description: OptionalNullable[str] = UNSET
    r"""A description of the object."""

    gender: OptionalNullable[Gender] = UNSET
    r"""The gender represents the gender identity of a person."""

    pronouns: OptionalNullable[str] = UNSET
    r"""The preferred pronouns of the person."""

    preferred_language: OptionalNullable[str] = UNSET
    r"""language code according to ISO 639-1. For the United States - EN"""

    languages: Optional[List[Nullable[str]]] = None

    nationalities: Optional[List[Nullable[str]]] = None

    photo_url: OptionalNullable[str] = UNSET
    r"""The URL of the photo of a person."""

    timezone: OptionalNullable[str] = UNSET
    r"""The time zone related to the resource. The value is a string containing a standard time zone identifier, e.g. Europe/London."""

    source: OptionalNullable[str] = UNSET
    r"""When the employee is imported as a new hire, this field indicates what system (e.g. the name of the ATS) this employee was imported from."""

    source_id: OptionalNullable[str] = UNSET
    r"""Unique identifier of the employee in the system this employee was imported from (e.g. the ID in the ATS)."""

    record_url: OptionalNullable[str] = UNSET

    jobs: OptionalNullable[List[EmployeeJob]] = UNSET

    compensations: OptionalNullable[List[EmployeeCompensation]] = UNSET

    works_remote: OptionalNullable[bool] = UNSET
    r"""Indicates if the employee works from a remote location."""

    addresses: Optional[List[Address]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    emails: Optional[List[Email]] = None

    custom_fields: Optional[List[CustomField]] = None

    social_links: Optional[List[SocialLink]] = None

    bank_accounts: Optional[List[BankAccount]] = None

    tax_code: OptionalNullable[str] = UNSET

    tax_id: OptionalNullable[str] = UNSET

    dietary_preference: OptionalNullable[str] = UNSET
    r"""Indicate the employee's dietary preference."""

    food_allergies: OptionalNullable[List[str]] = UNSET
    r"""Indicate the employee's food allergies."""

    probation_period: Optional[ProbationPeriod] = None

    tags: OptionalNullable[List[str]] = UNSET

    custom_mappings: OptionalNullable[CustomMappings] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    deleted: OptionalNullable[bool] = UNSET
    r"""Flag to indicate if the object is deleted."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "downstream_id",
            "first_name",
            "last_name",
            "middle_name",
            "display_name",
            "preferred_name",
            "initials",
            "salutation",
            "title",
            "marital_status",
            "partner",
            "division",
            "division_id",
            "department",
            "department_id",
            "department_name",
            "team",
            "company_id",
            "company_name",
            "employment_start_date",
            "employment_end_date",
            "leaving_reason",
            "employee_number",
            "employment_status",
            "employment_role",
            "ethnicity",
            "manager",
            "direct_reports",
            "social_security_number",
            "birthday",
            "deceased_on",
            "country_of_birth",
            "description",
            "gender",
            "pronouns",
            "preferred_language",
            "languages",
            "nationalities",
            "photo_url",
            "timezone",
            "source",
            "source_id",
            "record_url",
            "jobs",
            "compensations",
            "works_remote",
            "addresses",
            "phone_numbers",
            "emails",
            "custom_fields",
            "social_links",
            "bank_accounts",
            "tax_code",
            "tax_id",
            "dietary_preference",
            "food_allergies",
            "probation_period",
            "tags",
            "custom_mappings",
            "row_version",
            "deleted",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
            "pass_through",
        ]
        nullable_fields = [
            "id",
            "downstream_id",
            "first_name",
            "last_name",
            "middle_name",
            "display_name",
            "preferred_name",
            "initials",
            "salutation",
            "title",
            "marital_status",
            "division",
            "division_id",
            "department",
            "department_id",
            "department_name",
            "team",
            "company_id",
            "company_name",
            "employment_start_date",
            "employment_end_date",
            "leaving_reason",
            "employee_number",
            "employment_status",
            "ethnicity",
            "direct_reports",
            "social_security_number",
            "birthday",
            "deceased_on",
            "country_of_birth",
            "description",
            "gender",
            "pronouns",
            "preferred_language",
            "photo_url",
            "timezone",
            "source",
            "source_id",
            "record_url",
            "jobs",
            "compensations",
            "works_remote",
            "tax_code",
            "tax_id",
            "dietary_preference",
            "food_allergies",
            "tags",
            "custom_mappings",
            "row_version",
            "deleted",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EmployeeInputTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for an object."""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the person."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the person."""
    middle_name: NotRequired[Nullable[str]]
    r"""Middle name of the person."""
    display_name: NotRequired[Nullable[str]]
    r"""The name used to display the employee, often a combination of their first and last names."""
    preferred_name: NotRequired[Nullable[str]]
    r"""The name the employee prefers to be addressed by, which may be different from their legal name."""
    initials: NotRequired[Nullable[str]]
    r"""The initials of the person, usually derived from their first, middle, and last names."""
    salutation: NotRequired[Nullable[str]]
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""
    title: NotRequired[Nullable[str]]
    r"""The job title of the person."""
    marital_status: NotRequired[Nullable[str]]
    r"""The marital status of the employee."""
    partner: NotRequired[PersonInputTypedDict]
    division: NotRequired[Nullable[str]]
    r"""The division the person is currently in. Usually a collection of departments or teams or regions."""
    division_id: NotRequired[Nullable[str]]
    r"""Unique identifier of the division this employee belongs to."""
    department: NotRequired[Nullable[str]]
    r"""The department the person is currently in. [Deprecated](https://developers.apideck.com/changelog) in favor of the dedicated department_id and department_name field."""
    department_id: NotRequired[Nullable[str]]
    r"""Unique identifier of the department ID this employee belongs to."""
    department_name: NotRequired[Nullable[str]]
    r"""Name of the department this employee belongs to."""
    team: NotRequired[Nullable[TeamTypedDict]]
    r"""The team the person is currently in."""
    company_id: NotRequired[Nullable[str]]
    r"""The unique identifier of the company."""
    company_name: NotRequired[Nullable[str]]
    r"""The name of the company."""
    employment_start_date: NotRequired[Nullable[str]]
    r"""A Start Date is the date that the employee started working at the company"""
    employment_end_date: NotRequired[Nullable[str]]
    r"""An End Date is the date that the employee ended working at the company"""
    leaving_reason: NotRequired[Nullable[LeavingReason]]
    r"""The reason because the employment ended."""
    employee_number: NotRequired[Nullable[str]]
    r"""An Employee Number, Employee ID or Employee Code, is a unique number that has been assigned to each individual staff member within a company."""
    employment_status: NotRequired[Nullable[EmploymentStatus]]
    r"""The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status."""
    employment_role: NotRequired[EmploymentRoleTypedDict]
    ethnicity: NotRequired[Nullable[str]]
    r"""The ethnicity of the employee"""
    manager: NotRequired[ManagerTypedDict]
    direct_reports: NotRequired[Nullable[List[str]]]
    r"""Direct reports is an array of ids that reflect the individuals in an organizational hierarchy who are directly supervised by this specific employee."""
    social_security_number: NotRequired[Nullable[str]]
    r"""A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions."""
    birthday: NotRequired[Nullable[date]]
    r"""The date of birth of the person."""
    deceased_on: NotRequired[Nullable[date]]
    r"""The date the person deceased."""
    country_of_birth: NotRequired[Nullable[str]]
    r"""Country code according to ISO 3166-1 alpha-2."""
    description: NotRequired[Nullable[str]]
    r"""A description of the object."""
    gender: NotRequired[Nullable[Gender]]
    r"""The gender represents the gender identity of a person."""
    pronouns: NotRequired[Nullable[str]]
    r"""The preferred pronouns of the person."""
    preferred_language: NotRequired[Nullable[str]]
    r"""language code according to ISO 639-1. For the United States - EN"""
    languages: NotRequired[List[Nullable[str]]]
    nationalities: NotRequired[List[Nullable[str]]]
    photo_url: NotRequired[Nullable[str]]
    r"""The URL of the photo of a person."""
    timezone: NotRequired[Nullable[str]]
    r"""The time zone related to the resource. The value is a string containing a standard time zone identifier, e.g. Europe/London."""
    source: NotRequired[Nullable[str]]
    r"""When the employee is imported as a new hire, this field indicates what system (e.g. the name of the ATS) this employee was imported from."""
    source_id: NotRequired[Nullable[str]]
    r"""Unique identifier of the employee in the system this employee was imported from (e.g. the ID in the ATS)."""
    record_url: NotRequired[Nullable[str]]
    jobs: NotRequired[Nullable[List[EmployeeJobInputTypedDict]]]
    compensations: NotRequired[Nullable[List[EmployeeCompensationInputTypedDict]]]
    works_remote: NotRequired[Nullable[bool]]
    r"""Indicates if the employee works from a remote location."""
    addresses: NotRequired[List[AddressTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    emails: NotRequired[List[EmailTypedDict]]
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    social_links: NotRequired[List[SocialLinkTypedDict]]
    bank_accounts: NotRequired[List[BankAccountTypedDict]]
    tax_code: NotRequired[Nullable[str]]
    tax_id: NotRequired[Nullable[str]]
    dietary_preference: NotRequired[Nullable[str]]
    r"""Indicate the employee's dietary preference."""
    food_allergies: NotRequired[Nullable[List[str]]]
    r"""Indicate the employee's food allergies."""
    probation_period: NotRequired[ProbationPeriodTypedDict]
    tags: NotRequired[Nullable[List[str]]]
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    deleted: NotRequired[Nullable[bool]]
    r"""Flag to indicate if the object is deleted."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class EmployeeInput(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for an object."""

    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the person."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the person."""

    middle_name: OptionalNullable[str] = UNSET
    r"""Middle name of the person."""

    display_name: OptionalNullable[str] = UNSET
    r"""The name used to display the employee, often a combination of their first and last names."""

    preferred_name: OptionalNullable[str] = UNSET
    r"""The name the employee prefers to be addressed by, which may be different from their legal name."""

    initials: OptionalNullable[str] = UNSET
    r"""The initials of the person, usually derived from their first, middle, and last names."""

    salutation: OptionalNullable[str] = UNSET
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""

    title: OptionalNullable[str] = UNSET
    r"""The job title of the person."""

    marital_status: OptionalNullable[str] = UNSET
    r"""The marital status of the employee."""

    partner: Optional[PersonInput] = None

    division: OptionalNullable[str] = UNSET
    r"""The division the person is currently in. Usually a collection of departments or teams or regions."""

    division_id: OptionalNullable[str] = UNSET
    r"""Unique identifier of the division this employee belongs to."""

    department: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""The department the person is currently in. [Deprecated](https://developers.apideck.com/changelog) in favor of the dedicated department_id and department_name field."""

    department_id: OptionalNullable[str] = UNSET
    r"""Unique identifier of the department ID this employee belongs to."""

    department_name: OptionalNullable[str] = UNSET
    r"""Name of the department this employee belongs to."""

    team: OptionalNullable[Team] = UNSET
    r"""The team the person is currently in."""

    company_id: OptionalNullable[str] = UNSET
    r"""The unique identifier of the company."""

    company_name: OptionalNullable[str] = UNSET
    r"""The name of the company."""

    employment_start_date: OptionalNullable[str] = UNSET
    r"""A Start Date is the date that the employee started working at the company"""

    employment_end_date: OptionalNullable[str] = UNSET
    r"""An End Date is the date that the employee ended working at the company"""

    leaving_reason: OptionalNullable[LeavingReason] = UNSET
    r"""The reason because the employment ended."""

    employee_number: OptionalNullable[str] = UNSET
    r"""An Employee Number, Employee ID or Employee Code, is a unique number that has been assigned to each individual staff member within a company."""

    employment_status: OptionalNullable[EmploymentStatus] = UNSET
    r"""The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status."""

    employment_role: Optional[EmploymentRole] = None

    ethnicity: OptionalNullable[str] = UNSET
    r"""The ethnicity of the employee"""

    manager: Optional[Manager] = None

    direct_reports: OptionalNullable[List[str]] = UNSET
    r"""Direct reports is an array of ids that reflect the individuals in an organizational hierarchy who are directly supervised by this specific employee."""

    social_security_number: OptionalNullable[str] = UNSET
    r"""A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions."""

    birthday: OptionalNullable[date] = UNSET
    r"""The date of birth of the person."""

    deceased_on: OptionalNullable[date] = UNSET
    r"""The date the person deceased."""

    country_of_birth: OptionalNullable[str] = UNSET
    r"""Country code according to ISO 3166-1 alpha-2."""

    description: OptionalNullable[str] = UNSET
    r"""A description of the object."""

    gender: OptionalNullable[Gender] = UNSET
    r"""The gender represents the gender identity of a person."""

    pronouns: OptionalNullable[str] = UNSET
    r"""The preferred pronouns of the person."""

    preferred_language: OptionalNullable[str] = UNSET
    r"""language code according to ISO 639-1. For the United States - EN"""

    languages: Optional[List[Nullable[str]]] = None

    nationalities: Optional[List[Nullable[str]]] = None

    photo_url: OptionalNullable[str] = UNSET
    r"""The URL of the photo of a person."""

    timezone: OptionalNullable[str] = UNSET
    r"""The time zone related to the resource. The value is a string containing a standard time zone identifier, e.g. Europe/London."""

    source: OptionalNullable[str] = UNSET
    r"""When the employee is imported as a new hire, this field indicates what system (e.g. the name of the ATS) this employee was imported from."""

    source_id: OptionalNullable[str] = UNSET
    r"""Unique identifier of the employee in the system this employee was imported from (e.g. the ID in the ATS)."""

    record_url: OptionalNullable[str] = UNSET

    jobs: OptionalNullable[List[EmployeeJobInput]] = UNSET

    compensations: OptionalNullable[List[EmployeeCompensationInput]] = UNSET

    works_remote: OptionalNullable[bool] = UNSET
    r"""Indicates if the employee works from a remote location."""

    addresses: Optional[List[Address]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    emails: Optional[List[Email]] = None

    custom_fields: Optional[List[CustomField]] = None

    social_links: Optional[List[SocialLink]] = None

    bank_accounts: Optional[List[BankAccount]] = None

    tax_code: OptionalNullable[str] = UNSET

    tax_id: OptionalNullable[str] = UNSET

    dietary_preference: OptionalNullable[str] = UNSET
    r"""Indicate the employee's dietary preference."""

    food_allergies: OptionalNullable[List[str]] = UNSET
    r"""Indicate the employee's food allergies."""

    probation_period: Optional[ProbationPeriod] = None

    tags: OptionalNullable[List[str]] = UNSET

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    deleted: OptionalNullable[bool] = UNSET
    r"""Flag to indicate if the object is deleted."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "first_name",
            "last_name",
            "middle_name",
            "display_name",
            "preferred_name",
            "initials",
            "salutation",
            "title",
            "marital_status",
            "partner",
            "division",
            "division_id",
            "department",
            "department_id",
            "department_name",
            "team",
            "company_id",
            "company_name",
            "employment_start_date",
            "employment_end_date",
            "leaving_reason",
            "employee_number",
            "employment_status",
            "employment_role",
            "ethnicity",
            "manager",
            "direct_reports",
            "social_security_number",
            "birthday",
            "deceased_on",
            "country_of_birth",
            "description",
            "gender",
            "pronouns",
            "preferred_language",
            "languages",
            "nationalities",
            "photo_url",
            "timezone",
            "source",
            "source_id",
            "record_url",
            "jobs",
            "compensations",
            "works_remote",
            "addresses",
            "phone_numbers",
            "emails",
            "custom_fields",
            "social_links",
            "bank_accounts",
            "tax_code",
            "tax_id",
            "dietary_preference",
            "food_allergies",
            "probation_period",
            "tags",
            "row_version",
            "deleted",
            "pass_through",
        ]
        nullable_fields = [
            "id",
            "first_name",
            "last_name",
            "middle_name",
            "display_name",
            "preferred_name",
            "initials",
            "salutation",
            "title",
            "marital_status",
            "division",
            "division_id",
            "department",
            "department_id",
            "department_name",
            "team",
            "company_id",
            "company_name",
            "employment_start_date",
            "employment_end_date",
            "leaving_reason",
            "employee_number",
            "employment_status",
            "ethnicity",
            "direct_reports",
            "social_security_number",
            "birthday",
            "deceased_on",
            "country_of_birth",
            "description",
            "gender",
            "pronouns",
            "preferred_language",
            "photo_url",
            "timezone",
            "source",
            "source_id",
            "record_url",
            "jobs",
            "compensations",
            "works_remote",
            "tax_code",
            "tax_id",
            "dietary_preference",
            "food_allergies",
            "tags",
            "row_version",
            "deleted",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .getconnectorresourceresponse import (
    GetConnectorResourceResponse,
    GetConnectorResourceResponseTypedDict,
)
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from .unexpectederrorresponse import (
    UnexpectedErrorResponse,
    UnexpectedErrorResponseTypedDict,
)
from .unifiedapiid import UnifiedAPIID
from apideck_unify.types import BaseModel
from apideck_unify.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ConnectorConnectorResourcesOneGlobalsTypedDict(TypedDict):
    app_id: NotRequired[str]
    r"""The ID of your Unify application"""


class ConnectorConnectorResourcesOneGlobals(BaseModel):
    app_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-app-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of your Unify application"""


class ConnectorConnectorResourcesOneRequestTypedDict(TypedDict):
    id: str
    r"""ID of the record you are acting upon."""
    resource_id: str
    r"""ID of the resource you are acting upon."""
    app_id: NotRequired[str]
    r"""The ID of your Unify application"""
    unified_api: NotRequired[UnifiedAPIID]
    r"""Specify unified API for the connector resource. This is useful when a resource appears in multiple APIs"""


class ConnectorConnectorResourcesOneRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""ID of the record you are acting upon."""

    resource_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""ID of the resource you are acting upon."""

    app_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-app-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of your Unify application"""

    unified_api: Annotated[
        Optional[UnifiedAPIID],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Specify unified API for the connector resource. This is useful when a resource appears in multiple APIs"""


class ConnectorConnectorResourcesOneResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    get_connector_resource_response: NotRequired[GetConnectorResourceResponseTypedDict]
    r"""ConnectorResources"""
    unexpected_error_response: NotRequired[UnexpectedErrorResponseTypedDict]
    r"""Unexpected error"""


class ConnectorConnectorResourcesOneResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    get_connector_resource_response: Optional[GetConnectorResourceResponse] = None
    r"""ConnectorResources"""

    unexpected_error_response: Optional[UnexpectedErrorResponse] = None
    r"""Unexpected error"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .creditordebit import CreditOrDebit
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from enum import Enum
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class StatementStatus(str, Enum):
    r"""The current status of the bank feed statement."""

    PENDING = "pending"
    REJECTED = "rejected"
    SUCCESS = "success"


class BankFeedStatementTransactionType(str, Enum):
    r"""Type of transaction."""

    CREDIT = "credit"
    DEBIT = "debit"
    DEPOSIT = "deposit"
    TRANSFER = "transfer"
    PAYMENT = "payment"
    OTHER = "other"


class TransactionsTypedDict(TypedDict):
    posted_date: datetime
    r"""The date of the transaction - YYYY:MM::DDThh:mm:ss.sTZD"""
    amount: float
    r"""The amount of the transaction."""
    credit_or_debit: CreditOrDebit
    r"""Whether the amount is a credit or debit."""
    source_transaction_id: str
    r"""The ID of the source transaction."""
    description: NotRequired[str]
    r"""A description of the transaction."""
    counterparty: NotRequired[str]
    r"""The counterparty of the transaction."""
    reference: NotRequired[str]
    r"""The reference of the transaction."""
    transaction_type: NotRequired[BankFeedStatementTransactionType]
    r"""Type of transaction."""


class Transactions(BaseModel):
    posted_date: datetime
    r"""The date of the transaction - YYYY:MM::DDThh:mm:ss.sTZD"""

    amount: float
    r"""The amount of the transaction."""

    credit_or_debit: CreditOrDebit
    r"""Whether the amount is a credit or debit."""

    source_transaction_id: str
    r"""The ID of the source transaction."""

    description: Optional[str] = None
    r"""A description of the transaction."""

    counterparty: Optional[str] = None
    r"""The counterparty of the transaction."""

    reference: Optional[str] = None
    r"""The reference of the transaction."""

    transaction_type: Optional[BankFeedStatementTransactionType] = None
    r"""Type of transaction."""


class BankFeedStatementTypedDict(TypedDict):
    id: str
    r"""A unique identifier for an object."""
    bank_feed_account_id: NotRequired[str]
    r"""The ID of the bank feed account this statement belongs to."""
    status: NotRequired[StatementStatus]
    r"""The current status of the bank feed statement."""
    start_date: NotRequired[datetime]
    r"""Start date of the bank feed statement."""
    end_date: NotRequired[datetime]
    r"""End date of the bank feed statement."""
    start_balance: NotRequired[float]
    r"""Balance amount at the start of the period."""
    start_balance_credit_or_debit: NotRequired[CreditOrDebit]
    r"""Whether the amount is a credit or debit."""
    end_balance: NotRequired[float]
    r"""Balance amount at the end of the period."""
    end_balance_credit_or_debit: NotRequired[CreditOrDebit]
    r"""Whether the amount is a credit or debit."""
    transactions: NotRequired[List[TransactionsTypedDict]]
    r"""List of transactions in the bank feed statement."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""


class BankFeedStatement(BaseModel):
    id: str
    r"""A unique identifier for an object."""

    bank_feed_account_id: Optional[str] = None
    r"""The ID of the bank feed account this statement belongs to."""

    status: Optional[StatementStatus] = None
    r"""The current status of the bank feed statement."""

    start_date: Optional[datetime] = None
    r"""Start date of the bank feed statement."""

    end_date: Optional[datetime] = None
    r"""End date of the bank feed statement."""

    start_balance: Optional[float] = None
    r"""Balance amount at the start of the period."""

    start_balance_credit_or_debit: Optional[CreditOrDebit] = None
    r"""Whether the amount is a credit or debit."""

    end_balance: Optional[float] = None
    r"""Balance amount at the end of the period."""

    end_balance_credit_or_debit: Optional[CreditOrDebit] = None
    r"""Whether the amount is a credit or debit."""

    transactions: Optional[List[Transactions]] = None
    r"""List of transactions in the bank feed statement."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "bank_feed_account_id",
            "status",
            "start_date",
            "end_date",
            "start_balance",
            "start_balance_credit_or_debit",
            "end_balance",
            "end_balance_credit_or_debit",
            "transactions",
            "created_at",
            "created_by",
            "updated_at",
            "updated_by",
        ]
        nullable_fields = ["created_at", "created_by", "updated_at", "updated_by"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class BankFeedStatementInputTypedDict(TypedDict):
    bank_feed_account_id: NotRequired[str]
    r"""The ID of the bank feed account this statement belongs to."""
    status: NotRequired[StatementStatus]
    r"""The current status of the bank feed statement."""
    start_date: NotRequired[datetime]
    r"""Start date of the bank feed statement."""
    end_date: NotRequired[datetime]
    r"""End date of the bank feed statement."""
    start_balance: NotRequired[float]
    r"""Balance amount at the start of the period."""
    start_balance_credit_or_debit: NotRequired[CreditOrDebit]
    r"""Whether the amount is a credit or debit."""
    end_balance: NotRequired[float]
    r"""Balance amount at the end of the period."""
    end_balance_credit_or_debit: NotRequired[CreditOrDebit]
    r"""Whether the amount is a credit or debit."""
    transactions: NotRequired[List[TransactionsTypedDict]]
    r"""List of transactions in the bank feed statement."""


class BankFeedStatementInput(BaseModel):
    bank_feed_account_id: Optional[str] = None
    r"""The ID of the bank feed account this statement belongs to."""

    status: Optional[StatementStatus] = None
    r"""The current status of the bank feed statement."""

    start_date: Optional[datetime] = None
    r"""Start date of the bank feed statement."""

    end_date: Optional[datetime] = None
    r"""End date of the bank feed statement."""

    start_balance: Optional[float] = None
    r"""Balance amount at the start of the period."""

    start_balance_credit_or_debit: Optional[CreditOrDebit] = None
    r"""Whether the amount is a credit or debit."""

    end_balance: Optional[float] = None
    r"""Balance amount at the end of the period."""

    end_balance_credit_or_debit: Optional[CreditOrDebit] = None
    r"""Whether the amount is a credit or debit."""

    transactions: Optional[List[Transactions]] = None
    r"""List of transactions in the bank feed statement."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .custommappings import CustomMappings, CustomMappingsTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class CollectionTypedDict(TypedDict):
    id: str
    r"""A unique identifier for an object."""
    parent_id: NotRequired[Nullable[str]]
    r"""The collections's parent ID"""
    type: NotRequired[Nullable[str]]
    r"""The collections's type"""
    name: NotRequired[Nullable[str]]
    r"""Name of the collection"""
    description: NotRequired[Nullable[str]]
    r"""Description of the collection"""
    custom_mappings: NotRequired[Nullable[CustomMappingsTypedDict]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""


class Collection(BaseModel):
    id: str
    r"""A unique identifier for an object."""

    parent_id: OptionalNullable[str] = UNSET
    r"""The collections's parent ID"""

    type: OptionalNullable[str] = UNSET
    r"""The collections's type"""

    name: OptionalNullable[str] = UNSET
    r"""Name of the collection"""

    description: OptionalNullable[str] = UNSET
    r"""Description of the collection"""

    custom_mappings: OptionalNullable[CustomMappings] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "parent_id",
            "type",
            "name",
            "description",
            "custom_mappings",
            "updated_at",
            "created_at",
        ]
        nullable_fields = [
            "parent_id",
            "type",
            "name",
            "description",
            "custom_mappings",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .linkedinvoiceitem import LinkedInvoiceItem, LinkedInvoiceItemTypedDict
from .linkedledgeraccount import LinkedLedgerAccount, LinkedLedgerAccountTypedDict
from .linkedledgeraccount_input import (
    LinkedLedgerAccountInput,
    LinkedLedgerAccountInputTypedDict,
)
from .linkedtaxrate import LinkedTaxRate, LinkedTaxRateTypedDict
from .linkedtaxrate_input import LinkedTaxRateInput, LinkedTaxRateInputTypedDict
from .linkedtrackingcategory import (
    LinkedTrackingCategory,
    LinkedTrackingCategoryTypedDict,
)
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from enum import Enum
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class BillLineItemType(str, Enum):
    r"""Bill Line Item type"""

    EXPENSE_ITEM = "expense_item"
    EXPENSE_ACCOUNT = "expense_account"
    OTHER = "other"


class BillLineItemTypedDict(TypedDict):
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    row_id: NotRequired[str]
    r"""Row ID"""
    code: NotRequired[Nullable[str]]
    r"""User defined item code"""
    line_number: NotRequired[Nullable[int]]
    r"""Line number in the invoice"""
    description: NotRequired[Nullable[str]]
    r"""User defined description"""
    type: NotRequired[Nullable[BillLineItemType]]
    r"""Bill Line Item type"""
    tax_amount: NotRequired[Nullable[float]]
    r"""Tax amount"""
    total_amount: NotRequired[Nullable[float]]
    r"""Total amount of the line item"""
    quantity: NotRequired[Nullable[float]]
    unit_price: NotRequired[Nullable[float]]
    unit_of_measure: NotRequired[Nullable[str]]
    r"""Description of the unit type the item is sold as, ie: kg, hour."""
    discount_percentage: NotRequired[Nullable[float]]
    r"""Discount percentage applied to the line item when supported downstream."""
    discount_amount: NotRequired[Nullable[float]]
    r"""Discount amount applied to the line item when supported downstream."""
    location_id: NotRequired[Nullable[str]]
    r"""The ID of the location"""
    department_id: NotRequired[Nullable[str]]
    r"""The ID of the department"""
    item: NotRequired[LinkedInvoiceItemTypedDict]
    tax_rate: NotRequired[LinkedTaxRateTypedDict]
    ledger_account: NotRequired[Nullable[LinkedLedgerAccountTypedDict]]
    tracking_categories: NotRequired[
        Nullable[List[Nullable[LinkedTrackingCategoryTypedDict]]]
    ]
    r"""A list of linked tracking categories."""
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""


class BillLineItem(BaseModel):
    id: Optional[str] = None
    r"""A unique identifier for an object."""

    row_id: Optional[str] = None
    r"""Row ID"""

    code: OptionalNullable[str] = UNSET
    r"""User defined item code"""

    line_number: OptionalNullable[int] = UNSET
    r"""Line number in the invoice"""

    description: OptionalNullable[str] = UNSET
    r"""User defined description"""

    type: OptionalNullable[BillLineItemType] = UNSET
    r"""Bill Line Item type"""

    tax_amount: OptionalNullable[float] = UNSET
    r"""Tax amount"""

    total_amount: OptionalNullable[float] = UNSET
    r"""Total amount of the line item"""

    quantity: OptionalNullable[float] = UNSET

    unit_price: OptionalNullable[float] = UNSET

    unit_of_measure: OptionalNullable[str] = UNSET
    r"""Description of the unit type the item is sold as, ie: kg, hour."""

    discount_percentage: OptionalNullable[float] = UNSET
    r"""Discount percentage applied to the line item when supported downstream."""

    discount_amount: OptionalNullable[float] = UNSET
    r"""Discount amount applied to the line item when supported downstream."""

    location_id: OptionalNullable[str] = UNSET
    r"""The ID of the location"""

    department_id: OptionalNullable[str] = UNSET
    r"""The ID of the department"""

    item: Optional[LinkedInvoiceItem] = None

    tax_rate: Optional[LinkedTaxRate] = None

    ledger_account: OptionalNullable[LinkedLedgerAccount] = UNSET

    tracking_categories: OptionalNullable[List[Nullable[LinkedTrackingCategory]]] = (
        UNSET
    )
    r"""A list of linked tracking categories."""

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "row_id",
            "code",
            "line_number",
            "description",
            "type",
            "tax_amount",
            "total_amount",
            "quantity",
            "unit_price",
            "unit_of_measure",
            "discount_percentage",
            "discount_amount",
            "location_id",
            "department_id",
            "item",
            "tax_rate",
            "ledger_account",
            "tracking_categories",
            "row_version",
            "updated_by",
            "created_by",
            "created_at",
            "updated_at",
        ]
        nullable_fields = [
            "code",
            "line_number",
            "description",
            "type",
            "tax_amount",
            "total_amount",
            "quantity",
            "unit_price",
            "unit_of_measure",
            "discount_percentage",
            "discount_amount",
            "location_id",
            "department_id",
            "ledger_account",
            "tracking_categories",
            "row_version",
            "updated_by",
            "created_by",
            "created_at",
            "updated_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class BillLineItemInputTypedDict(TypedDict):
    row_id: NotRequired[str]
    r"""Row ID"""
    code: NotRequired[Nullable[str]]
    r"""User defined item code"""
    line_number: NotRequired[Nullable[int]]
    r"""Line number in the invoice"""
    description: NotRequired[Nullable[str]]
    r"""User defined description"""
    type: NotRequired[Nullable[BillLineItemType]]
    r"""Bill Line Item type"""
    tax_amount: NotRequired[Nullable[float]]
    r"""Tax amount"""
    total_amount: NotRequired[Nullable[float]]
    r"""Total amount of the line item"""
    quantity: NotRequired[Nullable[float]]
    unit_price: NotRequired[Nullable[float]]
    unit_of_measure: NotRequired[Nullable[str]]
    r"""Description of the unit type the item is sold as, ie: kg, hour."""
    discount_percentage: NotRequired[Nullable[float]]
    r"""Discount percentage applied to the line item when supported downstream."""
    discount_amount: NotRequired[Nullable[float]]
    r"""Discount amount applied to the line item when supported downstream."""
    location_id: NotRequired[Nullable[str]]
    r"""The ID of the location"""
    department_id: NotRequired[Nullable[str]]
    r"""The ID of the department"""
    item: NotRequired[LinkedInvoiceItemTypedDict]
    tax_rate: NotRequired[LinkedTaxRateInputTypedDict]
    ledger_account: NotRequired[Nullable[LinkedLedgerAccountInputTypedDict]]
    tracking_categories: NotRequired[
        Nullable[List[Nullable[LinkedTrackingCategoryTypedDict]]]
    ]
    r"""A list of linked tracking categories."""
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""


class BillLineItemInput(BaseModel):
    row_id: Optional[str] = None
    r"""Row ID"""

    code: OptionalNullable[str] = UNSET
    r"""User defined item code"""

    line_number: OptionalNullable[int] = UNSET
    r"""Line number in the invoice"""

    description: OptionalNullable[str] = UNSET
    r"""User defined description"""

    type: OptionalNullable[BillLineItemType] = UNSET
    r"""Bill Line Item type"""

    tax_amount: OptionalNullable[float] = UNSET
    r"""Tax amount"""

    total_amount: OptionalNullable[float] = UNSET
    r"""Total amount of the line item"""

    quantity: OptionalNullable[float] = UNSET

    unit_price: OptionalNullable[float] = UNSET

    unit_of_measure: OptionalNullable[str] = UNSET
    r"""Description of the unit type the item is sold as, ie: kg, hour."""

    discount_percentage: OptionalNullable[float] = UNSET
    r"""Discount percentage applied to the line item when supported downstream."""

    discount_amount: OptionalNullable[float] = UNSET
    r"""Discount amount applied to the line item when supported downstream."""

    location_id: OptionalNullable[str] = UNSET
    r"""The ID of the location"""

    department_id: OptionalNullable[str] = UNSET
    r"""The ID of the department"""

    item: Optional[LinkedInvoiceItem] = None

    tax_rate: Optional[LinkedTaxRateInput] = None

    ledger_account: OptionalNullable[LinkedLedgerAccountInput] = UNSET

    tracking_categories: OptionalNullable[List[Nullable[LinkedTrackingCategory]]] = (
        UNSET
    )
    r"""A list of linked tracking categories."""

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "row_id",
            "code",
            "line_number",
            "description",
            "type",
            "tax_amount",
            "total_amount",
            "quantity",
            "unit_price",
            "unit_of_measure",
            "discount_percentage",
            "discount_amount",
            "location_id",
            "department_id",
            "item",
            "tax_rate",
            "ledger_account",
            "tracking_categories",
            "row_version",
        ]
        nullable_fields = [
            "code",
            "line_number",
            "description",
            "type",
            "tax_amount",
            "total_amount",
            "quantity",
            "unit_price",
            "unit_of_measure",
            "discount_percentage",
            "discount_amount",
            "location_id",
            "department_id",
            "ledger_account",
            "tracking_categories",
            "row_version",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from enum import Enum
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ConnectorSettingType(str, Enum):
    TEXT = "text"
    CHECKBOX = "checkbox"
    TEL = "tel"
    EMAIL = "email"
    URL = "url"
    TEXTAREA = "textarea"
    SELECT = "select"
    FILTERED_SELECT = "filtered-select"
    MULTI_SELECT = "multi-select"
    DATETIME = "datetime"
    DATE = "date"
    TIME = "time"
    NUMBER = "number"
    PASSWORD = "password"


class ConnectorSettingTypedDict(TypedDict):
    id: NotRequired[str]
    label: NotRequired[str]
    type: NotRequired[ConnectorSettingType]


class ConnectorSetting(BaseModel):
    id: Optional[str] = None

    label: Optional[str] = None

    type: Optional[ConnectorSettingType] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .bankaccount import BankAccount, BankAccountTypedDict
from .currency import Currency
from .customfield import CustomField, CustomFieldTypedDict
from .custommappings import CustomMappings, CustomMappingsTypedDict
from .email import Email, EmailTypedDict
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from .sociallink import SocialLink, SocialLinkTypedDict
from .website import Website, WebsiteTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import date, datetime
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class CompanyRowTypeTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    name: NotRequired[Nullable[str]]


class CompanyRowType(BaseModel):
    id: OptionalNullable[str] = UNSET

    name: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "name"]
        nullable_fields = ["id", "name"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class CompanyTypedDict(TypedDict):
    name: Nullable[str]
    r"""Name of the company"""
    id: NotRequired[str]
    r"""Unique identifier for the company"""
    interaction_count: NotRequired[Nullable[int]]
    r"""Number of interactions"""
    owner_id: NotRequired[Nullable[str]]
    r"""Owner ID"""
    image: NotRequired[Nullable[str]]
    r"""The Image URL of the company"""
    description: NotRequired[Nullable[str]]
    r"""A description of the company"""
    vat_number: NotRequired[Nullable[str]]
    r"""The VAT number of the company"""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    status: NotRequired[Nullable[str]]
    r"""The status of the company"""
    fax: NotRequired[Nullable[str]]
    r"""The fax number of the company"""
    annual_revenue: NotRequired[Nullable[str]]
    r"""The annual revenue of the company"""
    number_of_employees: NotRequired[Nullable[str]]
    r"""Number of employees"""
    industry: NotRequired[Nullable[str]]
    r"""The industry represents the type of business the company is in."""
    ownership: NotRequired[Nullable[str]]
    r"""The ownership indicates the type of ownership of the company."""
    sales_tax_number: NotRequired[Nullable[str]]
    r"""A sales tax number is a unique number that identifies a company for tax purposes."""
    payee_number: NotRequired[Nullable[str]]
    r"""A payee number is a unique number that identifies a payee for tax purposes."""
    abn_or_tfn: NotRequired[Nullable[str]]
    r"""An ABN is necessary for operating a business, while a TFN (Tax File Number) is required for any person working in Australia."""
    abn_branch: NotRequired[Nullable[str]]
    r"""An ABN Branch (also known as a GST Branch) is used if part of your business needs to account for GST separately from its parent entity."""
    acn: NotRequired[Nullable[str]]
    r"""The Australian Company Number (ACN) is a nine digit number with the last digit being a check digit calculated using a modified modulus 10 calculation. ASIC has adopted a convention of always printing and displaying the ACN in the format XXX XXX XXX; three blocks of three characters, each block separated by a blank."""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the person."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the person."""
    parent_id: NotRequired[Nullable[str]]
    r"""Parent ID"""
    bank_accounts: NotRequired[List[BankAccountTypedDict]]
    websites: NotRequired[List[WebsiteTypedDict]]
    addresses: NotRequired[List[AddressTypedDict]]
    social_links: NotRequired[List[SocialLinkTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    emails: NotRequired[List[EmailTypedDict]]
    row_type: NotRequired[CompanyRowTypeTypedDict]
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    tags: NotRequired[Nullable[List[str]]]
    read_only: NotRequired[Nullable[bool]]
    r"""Whether the company is read-only or not"""
    last_activity_at: NotRequired[Nullable[datetime]]
    r"""Last activity date"""
    deleted: NotRequired[bool]
    r"""Whether the company is deleted or not"""
    salutation: NotRequired[Nullable[str]]
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""
    birthday: NotRequired[Nullable[date]]
    r"""The date of birth of the person."""
    custom_mappings: NotRequired[Nullable[CustomMappingsTypedDict]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    updated_by: NotRequired[Nullable[str]]
    r"""Updated by user ID"""
    created_by: NotRequired[Nullable[str]]
    r"""Created by user ID"""
    updated_at: NotRequired[Nullable[datetime]]
    r"""Last updated date"""
    created_at: NotRequired[Nullable[datetime]]
    r"""Creation date"""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class Company(BaseModel):
    name: Nullable[str]
    r"""Name of the company"""

    id: Optional[str] = None
    r"""Unique identifier for the company"""

    interaction_count: OptionalNullable[int] = UNSET
    r"""Number of interactions"""

    owner_id: OptionalNullable[str] = UNSET
    r"""Owner ID"""

    image: OptionalNullable[str] = UNSET
    r"""The Image URL of the company"""

    description: OptionalNullable[str] = UNSET
    r"""A description of the company"""

    vat_number: OptionalNullable[str] = UNSET
    r"""The VAT number of the company"""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    status: OptionalNullable[str] = UNSET
    r"""The status of the company"""

    fax: OptionalNullable[str] = UNSET
    r"""The fax number of the company"""

    annual_revenue: OptionalNullable[str] = UNSET
    r"""The annual revenue of the company"""

    number_of_employees: OptionalNullable[str] = UNSET
    r"""Number of employees"""

    industry: OptionalNullable[str] = UNSET
    r"""The industry represents the type of business the company is in."""

    ownership: OptionalNullable[str] = UNSET
    r"""The ownership indicates the type of ownership of the company."""

    sales_tax_number: OptionalNullable[str] = UNSET
    r"""A sales tax number is a unique number that identifies a company for tax purposes."""

    payee_number: OptionalNullable[str] = UNSET
    r"""A payee number is a unique number that identifies a payee for tax purposes."""

    abn_or_tfn: OptionalNullable[str] = UNSET
    r"""An ABN is necessary for operating a business, while a TFN (Tax File Number) is required for any person working in Australia."""

    abn_branch: OptionalNullable[str] = UNSET
    r"""An ABN Branch (also known as a GST Branch) is used if part of your business needs to account for GST separately from its parent entity."""

    acn: OptionalNullable[str] = UNSET
    r"""The Australian Company Number (ACN) is a nine digit number with the last digit being a check digit calculated using a modified modulus 10 calculation. ASIC has adopted a convention of always printing and displaying the ACN in the format XXX XXX XXX; three blocks of three characters, each block separated by a blank."""

    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the person."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the person."""

    parent_id: OptionalNullable[str] = UNSET
    r"""Parent ID"""

    bank_accounts: Optional[List[BankAccount]] = None

    websites: Optional[List[Website]] = None

    addresses: Optional[List[Address]] = None

    social_links: Optional[List[SocialLink]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    emails: Optional[List[Email]] = None

    row_type: Optional[CompanyRowType] = None

    custom_fields: Optional[List[CustomField]] = None

    tags: OptionalNullable[List[str]] = UNSET

    read_only: OptionalNullable[bool] = UNSET
    r"""Whether the company is read-only or not"""

    last_activity_at: OptionalNullable[datetime] = UNSET
    r"""Last activity date"""

    deleted: Optional[bool] = None
    r"""Whether the company is deleted or not"""

    salutation: OptionalNullable[str] = UNSET
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""

    birthday: OptionalNullable[date] = UNSET
    r"""The date of birth of the person."""

    custom_mappings: OptionalNullable[CustomMappings] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    updated_by: OptionalNullable[str] = UNSET
    r"""Updated by user ID"""

    created_by: OptionalNullable[str] = UNSET
    r"""Created by user ID"""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""Last updated date"""

    created_at: OptionalNullable[datetime] = UNSET
    r"""Creation date"""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "interaction_count",
            "owner_id",
            "image",
            "description",
            "vat_number",
            "currency",
            "status",
            "fax",
            "annual_revenue",
            "number_of_employees",
            "industry",
            "ownership",
            "sales_tax_number",
            "payee_number",
            "abn_or_tfn",
            "abn_branch",
            "acn",
            "first_name",
            "last_name",
            "parent_id",
            "bank_accounts",
            "websites",
            "addresses",
            "social_links",
            "phone_numbers",
            "emails",
            "row_type",
            "custom_fields",
            "tags",
            "read_only",
            "last_activity_at",
            "deleted",
            "salutation",
            "birthday",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
            "pass_through",
        ]
        nullable_fields = [
            "name",
            "interaction_count",
            "owner_id",
            "image",
            "description",
            "vat_number",
            "currency",
            "status",
            "fax",
            "annual_revenue",
            "number_of_employees",
            "industry",
            "ownership",
            "sales_tax_number",
            "payee_number",
            "abn_or_tfn",
            "abn_branch",
            "acn",
            "first_name",
            "last_name",
            "parent_id",
            "tags",
            "read_only",
            "last_activity_at",
            "salutation",
            "birthday",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class CompanyInputTypedDict(TypedDict):
    name: Nullable[str]
    r"""Name of the company"""
    owner_id: NotRequired[Nullable[str]]
    r"""Owner ID"""
    image: NotRequired[Nullable[str]]
    r"""The Image URL of the company"""
    description: NotRequired[Nullable[str]]
    r"""A description of the company"""
    vat_number: NotRequired[Nullable[str]]
    r"""The VAT number of the company"""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    status: NotRequired[Nullable[str]]
    r"""The status of the company"""
    fax: NotRequired[Nullable[str]]
    r"""The fax number of the company"""
    annual_revenue: NotRequired[Nullable[str]]
    r"""The annual revenue of the company"""
    number_of_employees: NotRequired[Nullable[str]]
    r"""Number of employees"""
    industry: NotRequired[Nullable[str]]
    r"""The industry represents the type of business the company is in."""
    ownership: NotRequired[Nullable[str]]
    r"""The ownership indicates the type of ownership of the company."""
    sales_tax_number: NotRequired[Nullable[str]]
    r"""A sales tax number is a unique number that identifies a company for tax purposes."""
    payee_number: NotRequired[Nullable[str]]
    r"""A payee number is a unique number that identifies a payee for tax purposes."""
    abn_or_tfn: NotRequired[Nullable[str]]
    r"""An ABN is necessary for operating a business, while a TFN (Tax File Number) is required for any person working in Australia."""
    abn_branch: NotRequired[Nullable[str]]
    r"""An ABN Branch (also known as a GST Branch) is used if part of your business needs to account for GST separately from its parent entity."""
    acn: NotRequired[Nullable[str]]
    r"""The Australian Company Number (ACN) is a nine digit number with the last digit being a check digit calculated using a modified modulus 10 calculation. ASIC has adopted a convention of always printing and displaying the ACN in the format XXX XXX XXX; three blocks of three characters, each block separated by a blank."""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the person."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the person."""
    bank_accounts: NotRequired[List[BankAccountTypedDict]]
    websites: NotRequired[List[WebsiteTypedDict]]
    addresses: NotRequired[List[AddressTypedDict]]
    social_links: NotRequired[List[SocialLinkTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    emails: NotRequired[List[EmailTypedDict]]
    row_type: NotRequired[CompanyRowTypeTypedDict]
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    tags: NotRequired[Nullable[List[str]]]
    read_only: NotRequired[Nullable[bool]]
    r"""Whether the company is read-only or not"""
    salutation: NotRequired[Nullable[str]]
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""
    birthday: NotRequired[Nullable[date]]
    r"""The date of birth of the person."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class CompanyInput(BaseModel):
    name: Nullable[str]
    r"""Name of the company"""

    owner_id: OptionalNullable[str] = UNSET
    r"""Owner ID"""

    image: OptionalNullable[str] = UNSET
    r"""The Image URL of the company"""

    description: OptionalNullable[str] = UNSET
    r"""A description of the company"""

    vat_number: OptionalNullable[str] = UNSET
    r"""The VAT number of the company"""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    status: OptionalNullable[str] = UNSET
    r"""The status of the company"""

    fax: OptionalNullable[str] = UNSET
    r"""The fax number of the company"""

    annual_revenue: OptionalNullable[str] = UNSET
    r"""The annual revenue of the company"""

    number_of_employees: OptionalNullable[str] = UNSET
    r"""Number of employees"""

    industry: OptionalNullable[str] = UNSET
    r"""The industry represents the type of business the company is in."""

    ownership: OptionalNullable[str] = UNSET
    r"""The ownership indicates the type of ownership of the company."""

    sales_tax_number: OptionalNullable[str] = UNSET
    r"""A sales tax number is a unique number that identifies a company for tax purposes."""

    payee_number: OptionalNullable[str] = UNSET
    r"""A payee number is a unique number that identifies a payee for tax purposes."""

    abn_or_tfn: OptionalNullable[str] = UNSET
    r"""An ABN is necessary for operating a business, while a TFN (Tax File Number) is required for any person working in Australia."""

    abn_branch: OptionalNullable[str] = UNSET
    r"""An ABN Branch (also known as a GST Branch) is used if part of your business needs to account for GST separately from its parent entity."""

    acn: OptionalNullable[str] = UNSET
    r"""The Australian Company Number (ACN) is a nine digit number with the last digit being a check digit calculated using a modified modulus 10 calculation. ASIC has adopted a convention of always printing and displaying the ACN in the format XXX XXX XXX; three blocks of three characters, each block separated by a blank."""

    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the person."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the person."""

    bank_accounts: Optional[List[BankAccount]] = None

    websites: Optional[List[Website]] = None

    addresses: Optional[List[Address]] = None

    social_links: Optional[List[SocialLink]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    emails: Optional[List[Email]] = None

    row_type: Optional[CompanyRowType] = None

    custom_fields: Optional[List[CustomField]] = None

    tags: OptionalNullable[List[str]] = UNSET

    read_only: OptionalNullable[bool] = UNSET
    r"""Whether the company is read-only or not"""

    salutation: OptionalNullable[str] = UNSET
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""

    birthday: OptionalNullable[date] = UNSET
    r"""The date of birth of the person."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "owner_id",
            "image",
            "description",
            "vat_number",
            "currency",
            "status",
            "fax",
            "annual_revenue",
            "number_of_employees",
            "industry",
            "ownership",
            "sales_tax_number",
            "payee_number",
            "abn_or_tfn",
            "abn_branch",
            "acn",
            "first_name",
            "last_name",
            "bank_accounts",
            "websites",
            "addresses",
            "social_links",
            "phone_numbers",
            "emails",
            "row_type",
            "custom_fields",
            "tags",
            "read_only",
            "salutation",
            "birthday",
            "pass_through",
        ]
        nullable_fields = [
            "name",
            "owner_id",
            "image",
            "description",
            "vat_number",
            "currency",
            "status",
            "fax",
            "annual_revenue",
            "number_of_employees",
            "industry",
            "ownership",
            "sales_tax_number",
            "payee_number",
            "abn_or_tfn",
            "abn_branch",
            "acn",
            "first_name",
            "last_name",
            "tags",
            "read_only",
            "salutation",
            "birthday",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AgedReportFilterTypedDict(TypedDict):
    customer_id: NotRequired[str]
    r"""Filter by customer id"""
    supplier_id: NotRequired[str]
    r"""Filter by supplier id"""
    report_as_of_date: NotRequired[str]
    r"""The cutoff date for considering transactions"""
    period_count: NotRequired[int]
    r"""Number of periods to split the aged creditors report into"""
    period_length: NotRequired[int]
    r"""Length of each period in days"""


class AgedReportFilter(BaseModel):
    customer_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Filter by customer id"""

    supplier_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Filter by supplier id"""

    report_as_of_date: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""The cutoff date for considering transactions"""

    period_count: Annotated[Optional[int], FieldMetadata(query=True)] = None
    r"""Number of periods to split the aged creditors report into"""

    period_length: Annotated[Optional[int], FieldMetadata(query=True)] = None
    r"""Length of each period in days"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from apideck_unify.utils import FieldMetadata
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WebhookEventLogsFilterServiceTypedDict(TypedDict):
    id: NotRequired[str]


class WebhookEventLogsFilterService(BaseModel):
    id: Annotated[Optional[str], FieldMetadata(query=True)] = None


class WebhookEventLogsFilterTypedDict(TypedDict):
    exclude_apis: NotRequired[Nullable[str]]
    service: NotRequired[Nullable[WebhookEventLogsFilterServiceTypedDict]]
    consumer_id: NotRequired[Nullable[str]]
    entity_type: NotRequired[Nullable[str]]
    event_type: NotRequired[Nullable[str]]


class WebhookEventLogsFilter(BaseModel):
    exclude_apis: Annotated[OptionalNullable[str], FieldMetadata(query=True)] = UNSET

    service: Annotated[
        OptionalNullable[WebhookEventLogsFilterService], FieldMetadata(query=True)
    ] = UNSET

    consumer_id: Annotated[OptionalNullable[str], FieldMetadata(query=True)] = UNSET

    entity_type: Annotated[OptionalNullable[str], FieldMetadata(query=True)] = UNSET

    event_type: Annotated[OptionalNullable[str], FieldMetadata(query=True)] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "exclude_apis",
            "service",
            "consumer_id",
            "entity_type",
            "event_type",
        ]
        nullable_fields = [
            "exclude_apis",
            "service",
            "consumer_id",
            "entity_type",
            "event_type",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
